/*  pdnmesh - a 2D finite element solver
    Copyright (C) 2001-2004 Sarod Yatawatta <sarod@users.sf.net>  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  $Id: contour.c,v 1.22 2004/04/11 17:54:35 sarod Exp $
*/

/* Werner Hoch provided code to produce color in EPS files
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include "types.h"


#define LEGEND_WIDTH 40 
#define LEGEND_HEIGHT DEFAULT_HEIGHT
#define LEGEND_BORDER 5

int contour_levels; /* no of contour levels */
int current_plotting_contour; /* number in z array */


/* contour plotting */
static int 
plot_contour(triangle *t, double potential)
{
  /* variables for contour plotting */
  int H,L,E;
  
  MY_INT p1,p2,p3;
  MY_DOUBLE x1,x2,y1,y2;

#ifdef DEBUG
  printf("contour for triangle %ld\n",t->n);
#endif
		/* initialize the following to ignore compiler warnings */
 p1=p2=p3=0;
	x1=x2=y1=y2=0;
  
  /* now determine the contours if any */
  /* first determine point potentials */
  E=H=L=0;
  if ( Mzz(t->p0,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p0,current_plotting_contour) < potential )
    L++;
  else
    H++;
  if ( Mzz(t->p1,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p1,current_plotting_contour) < potential )
    L++;
  else
    H++;
  if ( Mzz(t->p2,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p2,current_plotting_contour) < potential )
    L++;
  else
    H++;
  
  /* now according to the values of E,H and L */
  /* fit the general case */
  if ( E == 3 ) { /* 3 contours */
    /* we do not plot this */
#ifdef DEBUG
    glBegin(GL_LINE_LOOP);
    /* get first point */
    glVertex2f( (GLfloat)Mx(t->p0), (GLfloat)My(t->p0));
    /* get second point */
    glVertex2f( (GLfloat)Mx(t->p1), (GLfloat)My(t->p1));  
    /* get third point */
    glVertex2f( (GLfloat)Mx(t->p2), (GLfloat)My(t->p2));
    glEnd();
#endif
#ifdef DEBUG
    printf("3 lines\n");
#endif
    return(3);
  }
  if ( E == 2 ) { /* 1 line */
    /* generalize to p1,p2 */
    if ( Mzz(t->p0,current_plotting_contour) != potential ) {
      p1 = t->p1; p2 = t->p2;
    } 
    else if ( Mzz(t->p1,current_plotting_contour) != potential ) {
      p1 = t->p2; p2 = t->p0;
    } 
    else if ( Mzz(t->p2,current_plotting_contour) != potential ) {
      p1 = t->p0; p2 = t->p1;
    } 
    else
      return(-1); /*error*/
    /* now plot the general line */
     glBegin(GL_LINE_LOOP);
     /* get first point */
     glVertex2f( (GLfloat)Mx(p1), (GLfloat)My(p1));
    /* get second point */
    glVertex2f( (GLfloat)Mx(p2), (GLfloat)My(p2));  
    glEnd();
#ifdef DEBUG
    printf("1 line boundary\n");
#endif
    return(1);
  }
  if ( (E == 1) && ( H == 1 ) && ( L == 1 ) ) {
    /* againe one line generalize */
    if ( Mzz(t->p0,current_plotting_contour) == potential ) {
      p1 = t->p1; p2 = t->p2; p3 = t->p0;
    } 
    else if ( Mzz(t->p1,current_plotting_contour) == potential ) {
      p1 = t->p2; p2 = t->p0; p3 = t->p1;
    } 
    else if ( Mzz(t->p2,current_plotting_contour) == potential ) {
      p1 = t->p0; p2 = t->p1; p3 = t->p2;
    } 
    else 
      return(-1);
    /* now interpolate p1,p2 to find the correct point */
    if ( Mzz(p1,current_plotting_contour) == Mzz(p2,current_plotting_contour) )
      return(-1); /* error */
    x1 = Mx(p2) 
      +( Mx(p1)-Mx(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    y1 = My(p2)
      +( My(p1)-My(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    
    /* now draw line between p3,(x1,y1) */
    glBegin(GL_LINE_LOOP);
    /* get first point */
    glVertex2f( (GLfloat)Mx(p3), (GLfloat)My(p3));
    glVertex2f( (GLfloat)x1,(GLfloat) y1 );
    glEnd();
#ifdef DEBUG
    printf("1 line equal\n");
#endif
    return(1);
  }
  
  if (( ( H == 2 ) && ( L ==1 ) ) || ( ( H == 1 ) && ( L == 2 ))) {
    /* two lines, need to generalize */
    
    if ( ( H == 2 ) && ( L ==1 ) ) {
      /* find the lowest point, assign it to p1 */
      if ( Mzz(t->p0,current_plotting_contour) < potential ) {
	p1 = t->p0; p2 = t->p1; p3 = t->p2;
      } else  
      if ( Mzz(t->p1,current_plotting_contour) < potential ) {
	p1 = t->p1; p2 = t->p2; p3 = t->p0;
      } else
      if ( Mzz(t->p2,current_plotting_contour) < potential ) {
	p1 = t->p2; p2 = t->p0; p3 = t->p1;
      } else 
	return(-1); /* error */
#ifdef DEBUG
      printf("1 line case1\n");
#endif
    }
    if ( ( H == 1 ) && ( L == 2 ) ) {
      /* find the highest point, assign it to p1 */
      if ( Mzz(t->p0,current_plotting_contour) > potential ) {
	p1 = t->p0; p2 = t->p1; p3 = t->p2;
      } else
      if ( Mzz(t->p1,current_plotting_contour) > potential ) {
	p1 = t->p1; p2 = t->p2; p3 = t->p0;
      } else
      if ( Mzz(t->p2,current_plotting_contour) > potential ) {
	p1 = t->p2; p2 = t->p0; p3 = t->p1;
      } else
	return(-1);
#ifdef DEBUG
      printf("1 line case2\n");
#endif
    }
    /* now interpolate between (p1,p2) and (p1,p3) */
  
    /* first p1,p2 interpolation */
    if ( Mzz(p1,current_plotting_contour) != Mzz(p2,current_plotting_contour) ) {
      x1 = Mx(p2) 
	+ (Mx(p1)-Mx(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
      y1 = My(p2)
	+ (My(p1)-My(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    }
    /* now p1,p3 interpolation */
    if ( Mzz(p1,current_plotting_contour) != Mzz(p3,current_plotting_contour) ) {
    x2 = Mx(p3) 
      + (Mx(p1)-Mx(p3))*(potential-Mzz(p3,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p3,current_plotting_contour));
    y2 = My(p3)
      + (My(p1)-My(p3))*(potential-Mzz(p3,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p3,current_plotting_contour));
  
    }
    /* now draw line (x1,y1) (x2,y2) */
    glBegin(GL_LINE_LOOP);
    /* get first point */
    glVertex2f( (GLfloat)x1, (GLfloat) y1 );
    glVertex2f( (GLfloat)x2, (GLfloat) y2 );
    glEnd();
    return(1); 

  }
  
  /* if we reach here, no contour */
  return(0);
}

/* 3d plot with potential */
static int 
plot_contour_in_3d(triangle *t, double potential)
{
  /* variables for contour plotting */
  int H,L,E;
  
  unsigned long int p1,p2,p3;
  long double x1,x2,y1,y2;
  double elevation;/*=potential/g_maxpot;*/
	if ( ABS(g_maxpot)<ABS(g_minpot) ) {
     elevation=potential/ABS(1.1*g_minpot);
	} else {
     elevation=potential/ABS(1.1*g_maxpot);
	}
#ifdef DEBUG
  printf("contour for triangle %ld\n",t->n);
#endif
  p1=p2=p3=0;
	 x1=x2=y1=y2=0;

  
  /* now determine the contours if any */
  /* first determine point potentials */
  E=H=L=0;
  if ( Mzz(t->p0,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p0,current_plotting_contour) < potential )
    L++;
  else
    H++;
  if ( Mzz(t->p1,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p1,current_plotting_contour) < potential )
    L++;
  else
    H++;
  if ( Mzz(t->p2,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p2,current_plotting_contour) < potential )
    L++;
  else
    H++;
  
  /* now according to the values of E,H and L */
  /* fit the general case */
  if ( E == 3 ) { /* 3 contours */
    /* we do not plot this */
#ifdef DEBUG
    glBegin(GL_LINE_LOOP);
    /* get first point */
    glVertex3f( (GLfloat)Mx(t->p0), (GLfloat)My(t->p0),(GLfloat)elevation);
    /* get second point */
    glVertex3f( (GLfloat)Mx(t->p1), (GLfloat)My(t->p1),(GLfloat)elevation);  
    /* get third point */
    glVertex3f( (GLfloat)Mx(t->p2), (GLfloat)My(t->p2),(GLfloat)elevation);
    glEnd();
#endif
#ifdef DEBUG
    printf("3 lines\n");
#endif
    return(3);
  }
  if ( E == 2 ) { /* 1 line */
    /* generalize to p1,p2 */
    if ( Mzz(t->p0,current_plotting_contour) != potential ) {
      p1 = t->p1; p2 = t->p2;
    } 
    else if ( Mzz(t->p1,current_plotting_contour) != potential ) {
      p1 = t->p2; p2 = t->p0;
    } 
    else if ( Mzz(t->p2,current_plotting_contour) != potential ) {
      p1 = t->p0; p2 = t->p1;
    } 
    else
      return(-1); /*error*/
    /* now plot the general line */
     glBegin(GL_LINE_LOOP);
     /* get first point */
     glVertex3f( (GLfloat)Mx(p1), (GLfloat)My(p1),(GLfloat)elevation);
    /* get second point */
    glVertex3f( (GLfloat)Mx(p2), (GLfloat)My(p2),(GLfloat)elevation);  
    glEnd();
#ifdef DEBUG
    printf("1 line boundary\n");
#endif
    return(1);
  }
  if ( (E == 1) && ( H == 1 ) && ( L == 1 ) ) {
    /* againe one line generalize */
    if ( Mzz(t->p0,current_plotting_contour) == potential ) {
      p1 = t->p1; p2 = t->p2; p3 = t->p0;
    } 
    else if ( Mzz(t->p1,current_plotting_contour) == potential ) {
      p1 = t->p2; p2 = t->p0; p3 = t->p1;
    } 
    else if ( Mzz(t->p2,current_plotting_contour) == potential ) {
      p1 = t->p0; p2 = t->p1; p3 = t->p2;
    } 
    else 
      return(-1);
    /* now interpolate p1,p2 to find the correct point */
    if ( Mzz(p1,current_plotting_contour) == Mzz(p2,current_plotting_contour) )
      return(-1); /* error */
    x1 = Mx(p2) 
      +( Mx(p1)-Mx(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    y1 = My(p2)
      +( My(p1)-My(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    
    /* now draw line between p3,(x1,y1) */
    glBegin(GL_LINE_LOOP);
    /* get first point */
    glVertex3f( (GLfloat)Mx(p3), (GLfloat)My(p3),(GLfloat)elevation);
    glVertex3f( (GLfloat)x1,(GLfloat) y1,(GLfloat)elevation );
    glEnd();
#ifdef DEBUG
    printf("1 line equal\n");
#endif
    return(1);
  }
  
  if (( ( H == 2 ) && ( L ==1 ) ) || ( ( H == 1 ) && ( L == 2 ))) {
    /* two lines, need to generalize */
    
    if ( ( H == 2 ) && ( L ==1 ) ) {
      /* find the lowest point, assign it to p1 */
      if ( Mzz(t->p0,current_plotting_contour) < potential ) {
	p1 = t->p0; p2 = t->p1; p3 = t->p2;
      } else  
      if ( Mzz(t->p1,current_plotting_contour) < potential ) {
	p1 = t->p1; p2 = t->p2; p3 = t->p0;
      } else
      if ( Mzz(t->p2,current_plotting_contour) < potential ) {
	p1 = t->p2; p2 = t->p0; p3 = t->p1;
      } else 
	return(-1); /* error */
#ifdef DEBUG
      printf("1 line case1\n");
#endif
    }
    if ( ( H == 1 ) && ( L == 2 ) ) {
      /* find the highest point, assign it to p1 */
      if ( Mzz(t->p0,current_plotting_contour) > potential ) {
	p1 = t->p0; p2 = t->p1; p3 = t->p2;
      } else
      if ( Mzz(t->p1,current_plotting_contour) > potential ) {
	p1 = t->p1; p2 = t->p2; p3 = t->p0;
      } else
      if ( Mzz(t->p2,current_plotting_contour) > potential ) {
	p1 = t->p2; p2 = t->p0; p3 = t->p1;
      } else
	return(-1);
#ifdef DEBUG
      printf("1 line case2\n");
#endif
    }
    /* now interpolate between (p1,p2) and (p1,p3) */
  
    /* first p1,p2 interpolation */
    if ( Mzz(p1,current_plotting_contour) != Mzz(p2,current_plotting_contour) ) {
      x1 = Mx(p2) 
	+ (Mx(p1)-Mx(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
      y1 = My(p2)
	+ (My(p1)-My(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    }
    /* now p1,p3 interpolation */
    if ( Mzz(p1,current_plotting_contour) != Mzz(p3,current_plotting_contour) ) {
    x2 = Mx(p3) 
      + (Mx(p1)-Mx(p3))*(potential-Mzz(p3,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p3,current_plotting_contour));
    y2 = My(p3)
      + (My(p1)-My(p3))*(potential-Mzz(p3,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p3,current_plotting_contour));
  
    }
    /* now draw line (x1,y1) (x2,y2) */
    glBegin(GL_LINE_LOOP);
    /* get first point */
    glVertex3f( (GLfloat)x1, (GLfloat) y1,(GLfloat)elevation );
    glVertex3f( (GLfloat)x2, (GLfloat) y2,(GLfloat)elevation );
    glEnd();
    return(1); 

  }
  
  /* if we reach here, no contour */
  return(0);
}

/* to get a color to plot */
static int 
get_colour(float* redp, float* greenp, float *bluep, int level, int max_levels)
{
       float cl  = (float)level/(float)max_levels;
/*
		 if ( cl < 0.166666667 ) {
            *redp = 0.0f;
            *greenp = 1.0f;
            *bluep = cl*6.0;
		 } else if ( cl < 0.333333333 ) {
             *redp = 0.0f;
            *greenp = 2.0f - 6.0*cl;
            *bluep = 1.0f;
		 } else if ( cl < 0.5 ) {
              *redp = cl*6.0-3.0;
            *greenp = 0.0f;
            *bluep = 1.0f;
		 } else if ( cl < 0.6666666667 ) {
               *redp = 1.0f;
            *greenp = 0.0f;
            *bluep = 4.0f-cl*6.0;
		 } else if ( cl < 0.833333333 ) {
            *redp = 1.0f;
            *greenp = cl*6.0 -4.0f;
            *bluep = 0.0f;
		 } else {
            *redp = 1.0f;
            *greenp = 1.0f;
            *bluep = cl*6.0-5.0f;
		 }
*/
		 if ( cl < 0.25 ) {
            *redp = 0.0f;
            *greenp = cl*4.0f;
            *bluep = 1.0;
		 } else if ( cl < 0.5 ) {
             *redp = 0.0f;
            *greenp = 1.0f;
            *bluep = 2.0f -cl*4.0f;
		 } else if ( cl < 0.75 ) {
              *redp = cl*4.0f-2.0f;
            *greenp = 1.0f;
            *bluep = 0.0f;
		 } else  {
               *redp = 1.0f;
            *greenp = 4.0f - cl*4.0f;
            *bluep = 0.0f;
		 }	

     /* blue - green */
		  /*
   if ( cl < 1.5 ) { 
   *redp = 0.5;
   *greenp = cl*0.5;
   *bluep = 1.0 - *greenp;
	*/
/* green - red */
		  /*
  } else  { 
   *redp = 0.25 + (cl-1.5)*0.5;
   *bluep = 0.0;
   *greenp = 1.0 - *redp;
  }
  */
  
		 /*
  *redp = (float)level/(float)max_levels;
  *greenp = 0.7f;
  *bluep = 1.0f - *redp;
  */
  return(0);

}

/* function to plot the contour */
void 
plot_contour_all(void)
{
  triangle *t;
  int j; /* for contour calculation */
  float r,g,b; /* colours */
  double step =  (g_maxpot-g_minpot)/(float)contour_levels; /* contour step */

  
  
#ifdef DEBUG
  printf("plotting from %lf to %lf at %lf intervals\n",g_maxpot,g_minpot,step);
#endif

		DAG_traverse_list_reset(&dt);
		t=DAG_traverse_prune_list(&dt);
		while(t) {
    if ( t && t->status==LIVE ) {
      for ( j = 0; j < contour_levels ; j++ ) {
	/* get the colour */
	get_colour(&r,&g,&b,j, contour_levels);
	/* set the colour */
	glColor3f((GLfloat)r, (GLfloat)g, (GLfloat)b);
	plot_contour(t,g_minpot+j*step);
      }
         
    }
				
		t=DAG_traverse_prune_list(&dt);
  }

}

void 
plot_contour_all_in_3d(void)
{
  triangle *t;
  int j; /* for contour calculation */
  float r,g,b; /* colours */
  double step =  (g_maxpot-g_minpot)/(float)contour_levels; /* contour step */

  
  
#ifdef DEBUG
  printf("plotting from %lf to %lf at %lf intervals\n",g_maxpot,g_minpot,step);
#endif

		DAG_traverse_list_reset(&dt);
		t=DAG_traverse_prune_list(&dt);
		while(t) {
    if ( t && t->status==LIVE ) {
      for ( j = 0; j < contour_levels ; j++ ) {
	/* get the colour */
	get_colour(&r,&g,&b,j, contour_levels);
	/* set the colour */
	glColor3f((GLfloat)r, (GLfloat)g, (GLfloat)b);
	plot_contour_in_3d(t,g_minpot+j*step);
      }
         
    }
				
		t=DAG_traverse_prune_list(&dt);
  }

}


/* a function to print the contour of a triangle to a postscript file */
static int 
print_contour(FILE *plotf, triangle *t, MY_DOUBLE potential, MY_DOUBLE myscale,
								float r, float g, float b, int color_flag)
{
/* variables for contour plotting */
  int H,L,E;
  
  MY_INT p1,p2,p3;
  MY_DOUBLE x1,x2,y1,y2;
  
  /* note: we use shorthand here as follows:
     /l {lineto} bind def 
     /m {moveto} bind def 
     /n {newpath} bind def 
     /s {stroke} bind def 
     /slw {setlinewidth} bind def 
  */


#ifdef DEBUG
  printf("contour for triangle %ld\n",t->n);
#endif
		/* initialize the following to ignore compiler warnings */
 p1=p2=p3=0;
x1=x2=y1=y2=0;
  
  /* now determine the contours if any */
  /* first determine point potentials */
  E=H=L=0;
  if ( Mzz(t->p0,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p0,current_plotting_contour) < potential )
    L++;
  else
    H++;
  if ( Mzz(t->p1,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p1,current_plotting_contour) < potential )
    L++;
  else
    H++;
  if ( Mzz(t->p2,current_plotting_contour) == potential )
    E++;
  else if ( Mzz(t->p2,current_plotting_contour) < potential )
    L++;
  else
    H++;
  
  /* now according to the values of E,H and L */
  /* fit the general case */
  if ( E == 3 ) { /* 3 contours */
    /* we do not plot this */
#ifdef DEBUG
    printf("3 lines\n");
#endif
    return(3);
  }
  if ( E == 2 ) { /* 1 line */
    /* generalize to p1,p2 */
    if ( Mzz(t->p0,current_plotting_contour) != potential ) {
      p1 = t->p1; p2 = t->p2;
    } 
    else if ( Mzz(t->p1,current_plotting_contour) != potential ) {
      p1 = t->p2; p2 = t->p0;
    } 
    else if ( Mzz(t->p2,current_plotting_contour) != potential ) {
      p1 = t->p0; p2 = t->p1;
    } 
    else
      return(-1); /*error*/
		/* set color */
		if ( color_flag )
			fprintf(plotf,"%f %f %f srgb\n",r,g,b);
   /* now plot the general line */
    fprintf(plotf,"n ");
    /* get first point */
    fprintf(plotf,"%lf %lf m ",(double) (g_xrange+Mx(p1)*g_xscale)*myscale, (double)(g_yrange + My(p1)*g_yscale)*myscale);
    /* get second point */
    fprintf(plotf,"%lf %lf l ",(double) (g_xrange+Mx(p2)*g_xscale)*myscale, (double)(g_yrange + My(p2)*g_yscale)*myscale);
    fprintf(plotf,"cp ");
    fprintf(plotf,"s\n");
#ifdef DEBUG
    printf("1 line boundary\n");
#endif
    return(1);
  }
  if ( (E == 1) && ( H == 1 ) && ( L == 1 ) ) {
    /* againe one line generalize */
    if ( Mzz(t->p0,current_plotting_contour) == potential ) {
      p1 = t->p1; p2 = t->p2; p3 = t->p0;
    } 
    else if ( Mzz(t->p1,current_plotting_contour) == potential ) {
      p1 = t->p2; p2 = t->p0; p3 = t->p1;
    } 
    else if ( Mzz(t->p2,current_plotting_contour) == potential ) {
      p1 = t->p0; p2 = t->p1; p3 = t->p2;
    } 
    else 
      return(-1);
    /* now interpolate p1,p2 to find the correct point */
    if ( Mzz(p1,current_plotting_contour) == Mzz(p2,current_plotting_contour) )
      return(-1); /* error */
    x1 = Mx(p2) 
      +( Mx(p1)-Mx(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    y1 = My(p2)
      +( My(p1)-My(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
 
			/* set color */
		if ( color_flag )
			fprintf(plotf,"%f %f %f srgb\n",r,g,b);
	
    /* now draw line between p3,(x1,y1) */
    fprintf(plotf,"n ");
    /* get first point */
    fprintf(plotf,"%lf %lf m ",(double) (g_xrange+Mx(p3)*g_xscale)*myscale, (double)(g_yrange + My(p3)*g_yscale)*myscale);
    /* get second point */
    fprintf(plotf,"%lf %lf l ",(double) (g_xrange+x1*g_xscale)*myscale, (double)(g_yrange + y1*g_yscale)*myscale);
    fprintf(plotf,"cp ");
    fprintf(plotf,"s\n");
   
#ifdef DEBUG
    printf("1 line equal\n");
#endif
    return(1);
  }
  
  if (( ( H == 2 ) && ( L ==1 ) ) || ( ( H == 1 ) && ( L == 2 ))) {
    /* two lines, need to generalize */
    
    if ( ( H == 2 ) && ( L ==1 ) ) {
      /* find the lowest point, assign it to p1 */
      if ( Mzz(t->p0,current_plotting_contour) < potential ) {
	p1 = t->p0; p2 = t->p1; p3 = t->p2;
      } else  
      if ( Mzz(t->p1,current_plotting_contour) < potential ) {
	p1 = t->p1; p2 = t->p2; p3 = t->p0;
      } else
      if ( Mzz(t->p2,current_plotting_contour) < potential ) {
	p1 = t->p2; p2 = t->p0; p3 = t->p1;
      } else 
	return(-1); /* error */
#ifdef DEBUG
      printf("1 line case1\n");
#endif
    }
    if ( ( H == 1 ) && ( L == 2 ) ) {
      /* find the highest point, assign it to p1 */
      if ( Mzz(t->p0,current_plotting_contour) > potential ) {
	p1 = t->p0; p2 = t->p1; p3 = t->p2;
      } else
      if ( Mzz(t->p1,current_plotting_contour) > potential ) {
	p1 = t->p1; p2 = t->p2; p3 = t->p0;
      } else
      if ( Mzz(t->p2,current_plotting_contour) > potential ) {
	p1 = t->p2; p2 = t->p0; p3 = t->p1;
      } else
	return(-1);
#ifdef DEBUG
      printf("1 line case2\n");
#endif
    }
    /* now interpolate between (p1,p2) and (p1,p3) */
  
    /* first p1,p2 interpolation */
    if ( Mzz(p1,current_plotting_contour) != Mzz(p2,current_plotting_contour) ) {
      x1 = Mx(p2) 
	+ (Mx(p1)-Mx(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
      y1 = My(p2)
	+ (My(p1)-My(p2))*(potential-Mzz(p2,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p2,current_plotting_contour));
    }
    /* now p1,p3 interpolation */
    if ( Mzz(p1,current_plotting_contour) != Mzz(p3,current_plotting_contour) ) {
    x2 = Mx(p3) 
      + (Mx(p1)-Mx(p3))*(potential-Mzz(p3,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p3,current_plotting_contour));
    y2 = My(p3)
      + (My(p1)-My(p3))*(potential-Mzz(p3,current_plotting_contour))/(Mzz(p1,current_plotting_contour)-Mzz(p3,current_plotting_contour));
  
    }
		/* set color */
		if ( color_flag )
			fprintf(plotf,"%f %f %f srgb\n",r,g,b);

    /* now draw line (x1,y1) (x2,y2) */
    fprintf(plotf,"n ");
    /* get first point */
    fprintf(plotf,"%lf %lf m ",(double) (g_xrange+x1*g_xscale)*myscale, (double)(g_yrange + y1*g_yscale)*myscale);
    /* get second point */
    fprintf(plotf,"%lf %lf l ",(double) (g_xrange+x2*g_xscale)*myscale, (double)(g_yrange + y2*g_yscale)*myscale);
    fprintf(plotf,"cp ");
    fprintf(plotf,"s\n");

  }
  
  /* if we reach here, no contour */
  return(0);

}

static void
print_polygon(polygon *poly, double myscale, FILE *plotf)
{
 
 if (poly&&poly->head) {
	int i=poly->nedges;
	poly_elist *tempe=poly->head;
  for (;i;i--) {
  fprintf(plotf,"n ");
  fprintf(plotf,"%lf %lf  m ",(double)(Mx(tempe->data.p1)*g_xscale+g_xrange)*myscale, (double)(My(tempe->data.p1)*g_yscale+g_yrange)*myscale);
  fprintf(plotf,"%lf %lf  l ",(double)(Mx(tempe->data.p2)*g_xscale+g_xrange)*myscale, (double)(My(tempe->data.p2)*g_yscale+g_yrange)*myscale);
   fprintf(plotf,"s\n");  
	 tempe=tempe->next;
  }
 }

}


/* printing contour plot */
void
print_contour_all(const char* filename,int color_flag)
{

	FILE *plotf;
  MY_DOUBLE myscale;
  int i;
	float r,g,b;
  triangle *t;
  double step =  (g_maxpot-g_minpot)/(float)contour_levels; /* contour step */

	if ((plotf=fopen(filename,"w"))==NULL){
			fprintf(stderr,"%s: %d: could not open file %s\n",__FILE__,__LINE__,filename);
	   return;	
	}


  /* determine the right bound of bounding box */
  if ( g_xrange > g_yrange )
    myscale = 400.0/g_xrange;
  else
    myscale = 400.0/g_yrange;

  /* print the header */
  fprintf(plotf,"%%!PS-Adobe-2.0 EPSF-3.0\n");
  fprintf(plotf,"%%%%Title: %s\n",filename);
  fprintf(plotf,"%%%%Generated By: "PACKAGE" "VERSION"  contour levels: %d, min= "MDF", max= "MDF"\n", contour_levels, g_minpot, g_maxpot);
  /* now bounding box -add border of 20 */
  fprintf(plotf,"%%%%BoundingBox: 0 0 %4.2lf %4.2lf\n",(double)(2*g_xrange*myscale+20), (double)(2*g_yrange*myscale+20) );
  fprintf(plotf,"%%%%Magnification: 1.0000\n");
  
  

  /* start plotting from 10 10 */
  fprintf(plotf,"10 10 translate\n");


  /* now print some shorter definitions */
  fprintf(plotf,"/l {lineto} bind def\n"); /* lineto */
  fprintf(plotf,"/m {moveto} bind def\n"); /* moveto */
  fprintf(plotf,"/n {newpath} bind def\n"); /* newpath */
  fprintf(plotf,"/s {stroke} bind def\n"); /* stroke */
  fprintf(plotf,"/slw {setlinewidth} bind def\n"); /* setlinewidth */
  fprintf(plotf,"/cp {closepath} bind def\n"); /* closepath */
	if ( color_flag )
	 fprintf(plotf,"/srgb {setrgbcolor} bind def\n"); /* rgb color */
  
  /* now print contours */
  fprintf(plotf,"1 slw\n");
 DAG_traverse_list_reset(&dt);
		t=DAG_traverse_prune_list(&dt);
		while(t) {
    if ( t && t->status==LIVE ) {
      for ( i = 0; i < contour_levels ; i++ ) {
	/* get the colour */
	get_colour(&r,&g,&b,i, contour_levels);
	/* set the colour */
	     print_contour(plotf,t,g_minpot+i*step,myscale,r,g,b,color_flag);
      }
         
    }
				
		t=DAG_traverse_prune_list(&dt);
  }
 
  /* then plot the boundaries */
  /* set the line thikness */
  fprintf(plotf,"2 slw\n");

	if ( color_flag )
   fprintf(plotf,"0 0 0 srgb\n");
  for ( i = 0; i < bound.npoly; i++ ) {
		print_polygon(&bound.poly_array[i],myscale,plotf);
	}
    fprintf(plotf,"%%%%EOF\n");
  
  fclose(plotf);
}

/* printing mesh as eps*/
void
print_mesh_eps(const char* filename)
{

	FILE *plotf;
  MY_DOUBLE myscale;
  int i;
  triangle *t;

	if ((plotf=fopen(filename,"w"))==NULL){
			fprintf(stderr,"%s: %d: could not open file %s\n",__FILE__,__LINE__,filename);
	   return;	
	}


  /* determine the right bound of bounding box */
  if ( g_xrange > g_yrange )
    myscale = 400.0/g_xrange;
  else
    myscale = 400.0/g_yrange;

  /* print the header */
  fprintf(plotf,"%%!PS-Adobe-2.0 EPSF-3.0\n");
  fprintf(plotf,"%%%%Title: %s\n",filename);
  fprintf(plotf,"%%%%Generated By: "PACKAGE" "VERSION"  contour levels: %d, min= "MDF", max= "MDF"\n", contour_levels, g_minpot, g_maxpot);
  /* now bounding box -add border of 20 */
  fprintf(plotf,"%%%%BoundingBox: 0 0 %4.2lf %4.2lf\n",(double)(2*g_xrange*myscale+20), (double)(2*g_yrange*myscale+20) );
  fprintf(plotf,"%%%%Magnification: 1.0000\n");

  

  /* start plotting from 10 10 */
  fprintf(plotf,"10 10 translate\n");


  /* now print some shorter definitions */
  fprintf(plotf,"/l {lineto} bind def\n"); /* lineto */
  fprintf(plotf,"/m {moveto} bind def\n"); /* moveto */
  fprintf(plotf,"/n {newpath} bind def\n"); /* newpath */
  fprintf(plotf,"/s {stroke} bind def\n"); /* stroke */
  fprintf(plotf,"/slw {setlinewidth} bind def\n"); /* setlinewidth */
  fprintf(plotf,"/cp {closepath} bind def\n"); /* closepath */
  
  /* now print contours */
  fprintf(plotf,"1 slw\n");
 DAG_traverse_list_reset(&dt);
		t=DAG_traverse_prune_list(&dt);
		while(t) {
    if ( t && t->status==LIVE ) {
	/* get the colour */
	/*get_colour(&r,&g,&b,j, contour_levels); */
	/* set the colour */
  fprintf(plotf,"n ");
  fprintf(plotf,"%lf %lf  m ",(double)(Mx(t->p0)*g_xscale+g_xrange)*myscale, (double)(My(t->p0)*g_yscale+g_yrange)*myscale);
  fprintf(plotf,"%lf %lf  l ",(double)(Mx(t->p1)*g_xscale+g_xrange)*myscale, (double)(My(t->p1)*g_yscale+g_yrange)*myscale);
   fprintf(plotf,"s\n");  
  fprintf(plotf,"n ");
  fprintf(plotf,"%lf %lf  m ",(double)(Mx(t->p1)*g_xscale+g_xrange)*myscale, (double)(My(t->p1)*g_yscale+g_yrange)*myscale);
  fprintf(plotf,"%lf %lf  l ",(double)(Mx(t->p2)*g_xscale+g_xrange)*myscale, (double)(My(t->p2)*g_yscale+g_yrange)*myscale);
   fprintf(plotf,"s\n");  
  fprintf(plotf,"n ");
  fprintf(plotf,"%lf %lf  m ",(double)(Mx(t->p2)*g_xscale+g_xrange)*myscale, (double)(My(t->p2)*g_yscale+g_yrange)*myscale);
  fprintf(plotf,"%lf %lf  l ",(double)(Mx(t->p0)*g_xscale+g_xrange)*myscale, (double)(My(t->p0)*g_yscale+g_yrange)*myscale);
   fprintf(plotf,"s\n");  
         
    }
				
		t=DAG_traverse_prune_list(&dt);
  }
 
  /* then plot the boundaries */
  /* set the line thikness */
  fprintf(plotf,"2 slw\n");
  for ( i = 0; i < bound.npoly; i++ ) {
		print_polygon(&bound.poly_array[i],myscale,plotf);
	}
    fprintf(plotf,"%%%%EOF\n");
  
  fclose(plotf);
}


/**********************************************/
/* legend plotting routines */
static void
realize(GtkWidget *widget, gpointer data)
{
	GdkGLContext *glcontext =gtk_widget_get_gl_context(widget);
	GdkGLDrawable *gldrawable =gtk_widget_get_gl_drawable(widget);
	if (!gdk_gl_drawable_gl_begin(gldrawable,glcontext))
					return;
	gdk_gl_drawable_gl_end(gldrawable);
}

static gboolean
configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer data)
{
		GdkGLContext *glcontext=gtk_widget_get_gl_context(widget);
		GdkGLDrawable *gldrawable=gtk_widget_get_gl_drawable(widget);
		GLfloat w=widget->allocation.width;
		GLfloat h=widget->allocation.height;
		if (!gdk_gl_drawable_gl_begin(gldrawable,glcontext))
						return(FALSE);
		glViewport(0,0,w,h);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluOrtho2D(-1,1,-1,1);
		glMatrixMode(GL_MODELVIEW);
		gdk_gl_drawable_gl_end(gldrawable);
		return(TRUE);
}

static gboolean
expose_event(GtkWidget *widget,
								GdkEventExpose *event,
								gpointer data)
{
  int i;
	float r,g,b;
	GLfloat dy; /* y step */
	GLfloat y1,y2,x1,x2,scale;
	GdkGLDrawable *gldrawable =gtk_widget_get_gl_drawable(widget);
       GdkGLContext *glcontext=gtk_widget_get_gl_context(widget);

  GLfloat w=(widget->allocation.width);
	GLfloat h=(widget->allocation.height);

if (!gdk_gl_drawable_gl_begin(gldrawable,glcontext))
   return(FALSE);

	/* rescale */
 if ( h >w )
		scale=1/h;
 else 
	  scale=1/w;
		w=2;
		h=2;

/* top, bottom border =h/20 */
 /* left border = w/20, color block=w/3, text=w/2, right border=w/15 */
	/* drawing */
	 dy=(h*0.9)/(GLfloat)contour_levels;

  y2=-0.45*h;
	y1=dy+y2;
  x1=-0.45*w;
	x2=-0.11*w;
   glClear(GL_COLOR_BUFFER_BIT);
#ifdef DEBUG
	printf("expose_ev: legend: rectangle (%f,%f)--(%f,%f)\n",x1,y1,x2,y2);
#endif


	/* horizontal line */
	glColor3f(0.0f,0.7f,0.7f);
	glBegin(GL_LINES);
	glVertex2d(x2,y2);
	glVertex2d(x2+0.5*w,y2);
	glEnd();

  for (i=0; i<contour_levels; i++) {
				get_colour(&r,&g,&b,i,contour_levels);
				glColor3f((GLfloat)r,(GLfloat)g,(GLfloat)b);
				glBegin(GL_QUADS);
				glVertex2d(x1,y1);
				glVertex2d(x2,y1);
				glVertex2d(x2,y2);
				glVertex2d(x1,y2);
				glEnd();

	/* horizontal line */
	glColor3f(0.0f,0.7f,0.7f);
	glBegin(GL_LINES);
	glVertex2d(x2,y1);
	glVertex2d(x2+0.5*w,y1);
	glEnd();

	y1+=dy;
	y2+=dy;

	}
if ( gdk_gl_drawable_is_double_buffered(gldrawable))
		gdk_gl_drawable_swap_buffers(gldrawable);
else
		glFlush();
gdk_gl_drawable_gl_end(gldrawable);
 return(TRUE);
}


static gint
close_legend_window(GtkWidget *widget, GdkEvent *event, gpointer *data)
{
  return(FALSE);
}

/* plotting legend */
void
display_legend_window(void)
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *drawing_area;
	GdkGLConfig *glconfig;



  /* Try double-buffered visual */
  glconfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
					GDK_GL_MODE_DOUBLE);
  if (glconfig == NULL)
    {
      g_print ("\n*** Cannot find the double-buffered visual.\n");
      g_print ("\n*** Trying single-buffered visual.\n");

      /* Try single-buffered visual */
      glconfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB);
      if (glconfig == NULL)
	{
	  g_print ("*** No appropriate OpenGL-capable visual found.\n");
	  exit(1);
	}
    }


  /*
   * Top-level window.
   */

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Legend");

  /* Get automatically redrawn if any of their children changed allocation. */
  gtk_container_set_reallocate_redraws (GTK_CONTAINER (window), TRUE);

  /* Connect signal handlers to the window */
  g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (gtk_widget_destroy), (gpointer)window);
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
              GTK_SIGNAL_FUNC (close_legend_window), NULL);

  /*
   * VBox.
   */

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_widget_show (vbox);

  /*
   * Drawing area to draw OpenGL scene.
   */

  drawing_area = gtk_drawing_area_new ();
  gtk_widget_set_size_request (drawing_area, LEGEND_WIDTH, LEGEND_HEIGHT);

  /* Set OpenGL-capability to the widget */
  gtk_widget_set_gl_capability (drawing_area,
				glconfig,
				NULL,
				TRUE,
				GDK_GL_RGBA_TYPE);

  gtk_widget_add_events (drawing_area,
			 GDK_BUTTON_PRESS_MASK      |
			 GDK_POINTER_MOTION_MASK    |  /* any pointer motion */
			 GDK_VISIBILITY_NOTIFY_MASK);

  /* Connect signal handlers to the drawing area */
  g_signal_connect_after (G_OBJECT (drawing_area), "realize",
                          G_CALLBACK (realize), NULL);
  g_signal_connect (G_OBJECT (drawing_area), "configure_event",
		    G_CALLBACK (configure_event), NULL);
  g_signal_connect (G_OBJECT (drawing_area), "expose_event",
		    G_CALLBACK (expose_event), NULL);


  gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);

  gtk_widget_show (drawing_area);


  gtk_widget_show(window);


}
